<?php

class Mbe_Shipping_Helper_Data extends Mage_Core_Helper_Abstract
{
    private $_mbeMediaDir = "mbe";
    const MBE_SHIPPING_PREFIX = "mbe_shipping_";

    //MAIN
    const XML_PATH_ENABLED = 'carriers/mbe_shipping/active';
    const XML_PATH_COUNTRY = 'carriers/mbe_shipping/country';
    //WS
    const XML_PATH_WS_URL = 'carriers/mbe_shipping/url';
    const XML_PATH_WS_USERNAME = 'carriers/mbe_shipping/username';
    const XML_PATH_WS_PASSWORD = 'carriers/mbe_shipping/password';
    //OPTIONS
    const XML_PATH_DESCRIPTION = 'carriers/mbe_shipping/description';
    const XML_PATH_DEFAULT_SHIPMENT_TYPE = 'carriers/mbe_shipping/default_shipment_type';
    const XML_PATH_ALLOWED_SHIPMENT_SERVICES = 'carriers/mbe_shipping/allowed_shipment_services';
    const XML_PATH_SHIPMENT_CONFIGURATION_MODE = 'carriers/mbe_shipping/shipment_configuration_mode';
    const XML_PATH_DEFAULT_LENGTH = 'carriers/mbe_shipping/default_length';
    const XML_PATH_DEFAULT_WIDTH = 'carriers/mbe_shipping/default_width';
    const XML_PATH_DEFAULT_HEIGHT = 'carriers/mbe_shipping/default_height';
    const XML_PATH_MAX_PACKAGE_WEIGHT = 'carriers/mbe_shipping/max_package_weight';
    const XML_PATH_MAX_SHIPMENT_WEIGHT = 'carriers/mbe_shipping/max_shipment_weight';
    const XML_PATH_HANDLING_TYPE = 'carriers/mbe_shipping/handling_type';
    const XML_PATH_HANDLING_ACTION = 'carriers/mbe_shipping/handling_action';
    const XML_PATH_HANDLING_FEE = 'carriers/mbe_shipping/handling_fee';
    const XML_PATH_HANDLING_FEE_ROUNDING = 'carriers/mbe_shipping/handling_fee_rounding';
    const XML_PATH_HANDLING_FEE_ROUNDING_AMOUNT = 'carriers/mbe_shipping/handling_fee_rounding_amount';
    const XML_PATH_SALLOWSPECIFIC = 'carriers/mbe_shipping/sallowspecific';
    const XML_PATH_SPECIFICCOUNTRY = 'carriers/mbe_shipping/specificcountry';
    const XML_PATH_SORT_ORDER = 'carriers/mbe_shipping/sort_order';
    const XML_PATH_MAXIMUM_TIME_FOR_SHIPPING_BEFORE_THE_END_OF_THE_DAY = 'carriers/mbe_shipping/maximum_time_for_shipping_before_the_end_of_the_day';
    const XML_PATH_SPECIFICERRMSG = 'carriers/mbe_shipping/specificerrmsg';
    const XML_PATH_WEIGHT_TYPE = 'carriers/mbe_shipping/weight_type';
    const XML_PATH_SHIPMENTS_CLOSURE_MODE = 'carriers/mbe_shipping/shipments_closure_mode';
    const XML_PATH_SHIPMENTS_CLOSURE_TIME = 'carriers/mbe_shipping/shipments_closure_time';

    const XML_PATH_THRESHOLD = 'carriers/mbe_shipping/threshold';

    //const

    const MBE_SHIPMENT_STATUS_CLOSED = "Closed";
    const MBE_SHIPMENT_STATUS_OPEN = "Opened";

    const MBE_CLOSURE_MODE_AUTOMATICALLY = 'automatically';
    const MBE_CLOSURE_MODE_MANUALLY = 'manually';

    public function checkMbeDir()
    {
        $mediaDir = Mage::getBaseDir('media');
        $mbeDir = $mediaDir . DS . 'mbe';
        if (!file_exists($mbeDir)) {
            mkdir($mbeDir);
        }
    }

    //TODO: check if is necessary to specify storeid or get current storeid
    public function isEnabled($storeId = null)
    {

        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');
        $result = $ws->isCustomerActive() && Mage::getStoreConfig(self::XML_PATH_ENABLED, $storeId);
        return $result;
    }

    public function getCountry($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_COUNTRY, $storeId);
    }


    public function getWsUrl($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_WS_URL, $storeId);
    }

    public function getWsUsername($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_WS_USERNAME, $storeId);
    }

    public function getWsPassword($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_WS_PASSWORD, $storeId);
    }

    public function getDescription($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_DESCRIPTION, $storeId);
    }

    public function getDefaultShipmentType($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_DEFAULT_SHIPMENT_TYPE, $storeId);
    }

    public function getAllowedShipmentServices($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_ALLOWED_SHIPMENT_SERVICES, $storeId);
    }

    public function getAllowedShipmentServicesArray($storeId = null)
    {
        $allowedShipmentServices = $this->getAllowedShipmentServices($storeId);

        $result = array();
        if ($allowedShipmentServices != "") {
            $result = explode(",", $allowedShipmentServices);
        }
        return $result;
    }

    public function getShipmentConfigurationMode($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SHIPMENT_CONFIGURATION_MODE, $storeId);
    }

    public function getDefaultLength($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_DEFAULT_LENGTH, $storeId);
    }

    public function getDefaultWidth($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_DEFAULT_WIDTH, $storeId);
    }

    public function getDefaultHeight($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_DEFAULT_HEIGHT, $storeId);
    }

    public function getMaxPackageWeight($storeId = null)
    {
        $result = Mage::getStoreConfig(self::XML_PATH_MAX_PACKAGE_WEIGHT, $storeId);
        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');

        if ($this->getDefaultShipmentType() == "ENVELOPE") {
            $maxParcelWeight = 0.5;
        }
        else {
            $maxParcelWeight = $ws->getCustomerPermission("maxParcelWeight");
        }

        if ($maxParcelWeight > 0 && $maxParcelWeight < $result) {
            $result = $maxParcelWeight;
        }
        return $result;
    }

    public function getMaxShipmentWeight($storeId = null)
    {
        $result = Mage::getStoreConfig(self::XML_PATH_MAX_SHIPMENT_WEIGHT, $storeId);
        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');

        if ($this->getDefaultShipmentType() == "ENVELOPE") {
            $maxShipmentWeight = 0.5;
        }
        else {
            $maxShipmentWeight = $ws->getCustomerPermission("maxShipmentWeight");
        }

        if ($maxShipmentWeight > 0 && $maxShipmentWeight < $result) {
            $result = $maxShipmentWeight;
        }
        return $result;
    }

    public function getHandlingType($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_HANDLING_TYPE, $storeId);
    }

    public function getHandlingAction($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_HANDLING_ACTION, $storeId);
    }

    public function getHandlingFee($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_HANDLING_FEE, $storeId);
    }

    public function getHandlingFeeRounding($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_HANDLING_FEE_ROUNDING, $storeId);
    }

    public function getHandlingFeeRoundingAmount($storeId = null)
    {
        $result = 1;
        if (Mage::getStoreConfig(self::XML_PATH_HANDLING_FEE_ROUNDING_AMOUNT, $storeId) == 2) {
            $result = 0.5;
        }
        return $result;
    }

    public function getSallowspecific($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SALLOWSPECIFIC, $storeId);
    }

    public function getSpecificcountry($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SPECIFICCOUNTRY, $storeId);
    }

    public function getSortOrder($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SORT_ORDER, $storeId);
    }

    public function getMaximumTimeForShippingBeforeTheEndOfTheDay($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_MAXIMUM_TIME_FOR_SHIPPING_BEFORE_THE_END_OF_THE_DAY, $storeId);
    }

    public function getSpecificerrmsg($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SPECIFICERRMSG, $storeId);
    }

    public function getWeightType($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_WEIGHT_TYPE, $storeId);
    }

    public function getShipmentsClosureMode($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SHIPMENTS_CLOSURE_MODE, $storeId);
    }

    public function getShipmentsClosureTime($storeId = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_SHIPMENTS_CLOSURE_TIME, $storeId);
    }


    public function round($value)
    {
        $result = $value;
        $handlingFeeRounding = $this->getHandlingFeeRounding();
        $handlingFeeRoundingAmount = $this->getHandlingFeeRoundingAmount();

        if ($handlingFeeRounding == 2) {

            if ($handlingFeeRoundingAmount == 1) {
                $result = round($value, 0);
            }
            else {
                $result = round($value, 2);
            }

        }
        elseif ($handlingFeeRounding == 3) {
            if ($handlingFeeRoundingAmount == 1) {
                $result = floor($value);
            }
            else {
                $result = floor($value * 2) / 2;
            }
        }
        elseif ($handlingFeeRounding == 4) {
            if ($handlingFeeRoundingAmount == 1) {
                $result = ceil($value);
            }
            else {
                $result = ceil($value * 2) / 2;
            }
        }
        return $result;
    }


    public function testRound($value)
    {
        $valueRounded = $this->round($value);
    }

    public function getNameFromLabel($label)
    {
        $name = $label;
        $name = strtolower($name);
        $name = str_replace(" ", "_", $name);
        $name = str_replace(" ", "_", $name);
        return $name;
    }

    public function getThresholdByShippingServrice($shippingService, $storeId = null)
    {
        $shippingService = strtolower($shippingService);
        //var_dump(self::XML_PATH_THRESHOLD . "_" . $shippingService);
        return Mage::getStoreConfig(self::XML_PATH_THRESHOLD . "_" . $shippingService, $storeId);
    }


    public function getTrackingStatus($trackingNumber)
    {
        $result = self::MBE_SHIPMENT_STATUS_OPEN;

        $mediaDir = Mage::getBaseDir('media');
        $mbeDir = $mediaDir . DS . 'mbe';
        $filePath = $mbeDir . DS . $trackingNumber . ".pdf";

        if (file_exists($filePath)) {
            $result = self::MBE_SHIPMENT_STATUS_CLOSED;
        }
        return $result;
    }

    public function isShippingOpen($shipmentId)
    {
        $result = true;
        $shipment = Mage::getModel("sales/order_shipment")->load($shipmentId);
        $tracks = $shipment->getAllTracks();
        foreach ($tracks as $track) {
            $trackingNumber = $track->getTrackNumber();
            $result = $result && $this->isTrackingOpen($trackingNumber);
        }
        return $result;
    }

    public function isTrackingOpen($trackingNumber)
    {
        return $this->getTrackingStatus($trackingNumber) == self::MBE_SHIPMENT_STATUS_OPEN;
    }


    public function getShipmentFilePath($shipmentIncrementId, $ext)
    {
        $this->checkMbeDir();
        $mediaDir = Mage::getBaseDir('media');
        $mbeDir = $mediaDir . DS . $this->_mbeMediaDir;

        $filePath = $mbeDir . DS . $shipmentIncrementId . "." . $ext;
        return $filePath;
    }

    public function getShipmentUrlByShipmentId($shipmentId)
    {
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);

        return $this->getShipmentUrl($shipment->getIncrementId());
    }

    public function getShipmentUrl($shipmentIncrementId)
    {
        $result = false;
        $mediaUrl = Mage::getBaseUrl('media');
        $mbeUrl = $mediaUrl . $this->_mbeMediaDir . '/';
        $ext = "pdf";

        if (file_exists($this->getShipmentFilePath($shipmentIncrementId, $ext))) {
            $result = $mbeUrl . $shipmentIncrementId . "." . $ext;
        }
        else {
            $ext = "html";
            if (file_exists($this->getShipmentFilePath($shipmentIncrementId, $ext))) {
                $result = $mbeUrl . $shipmentIncrementId . "." . $ext;
            }
        }
        return $result;
    }

    public function getTrackingFilePath($trackingNumber)
    {
        $this->checkMbeDir();
        $mediaDir = Mage::getBaseDir('media');
        $mbeDir = $mediaDir . DS . $this->_mbeMediaDir;
        $filePath = $mbeDir . DS . $trackingNumber . ".pdf";
        return $filePath;
    }

    public function getTrackingUrl($trackingNumber)
    {
        $mediaUrl = Mage::getBaseUrl('media');
        $mbeUrl = $mediaUrl . $this->_mbeMediaDir . '/';
        $fileUrl = $mbeUrl . $trackingNumber . ".pdf";
        return $fileUrl;
    }


    public function getShippingTypesFromShipments()
    {
        $result = array();

        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');
        $collection = Mage::getModel('sales/order')->getCollection()
            ->addFieldToSelect('shipping_method');

        $collection->getSelect()->group('shipping_method');

        foreach ($collection as $col) {
            $shippingMethodCode = $this->getShippingCodeFromMagentoShippingMethod($col->getShippingMethod());
            $result[$col->getShippingMethod()] = $ws->getLabelFromShipmentType($shippingMethodCode);
        }
        return $result;
    }


    public function getShipmentLabel($shipmentCode)
    {
        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');

        return $ws->getLabelFromShipmentType($shipmentCode);
    }

    public function getShipmentLabelFromMagentoShippingMethod($shipmentCode)
    {
        return $this->getShipmentLabel($this->getShippingCodeFromMagentoShippingMethod($shipmentCode));
    }


    public function isMbeShipping($order)
    {
        $shippingMethod = $order->getShippingMethod();
        if (strpos($shippingMethod, 'mbe_shipping_') !== false) {
            return true;
        }
        return false;
    }

    public function getMbeShippingType($order)
    {
        $result = "";
        $shippingMethod = $order->getShippingMethod();
        if (strpos($shippingMethod, self::MBE_SHIPPING_PREFIX) !== false) {

            $result = $this->getShippingCodeFromMagentoShippingMethod($shippingMethod);
        }
        return $result;
    }

    public function getShippingParamFromMagentoShippingMethod($shippingMethod, $i)
    {
        $shippingMethodWithSubZone = str_replace(self::MBE_SHIPPING_PREFIX, '', $shippingMethod);

        $shippingMethodWithSubZoneArray = explode('_', $shippingMethodWithSubZone);
        return $shippingMethodWithSubZoneArray[$i];
    }

    public function getShippingCodeFromMagentoShippingMethod($shippingMethod)
    {
        return $this->getShippingParamFromMagentoShippingMethod($shippingMethod, 0);
    }

    public function getShippingSubZoneFromMagentoShippingMethod($shippingMethod)
    {
        return $this->getShippingParamFromMagentoShippingMethod($shippingMethod, 1);
    }
}