<?php

class Mbe_Shipping_Adminhtml_ShipmentController extends Mage_Adminhtml_Controller_Action
{

    protected function _isAllowed()
    {
        return Mage::getSingleton('admin/session')->isAllowed('admin/sales/mbe_shippings');
    }

    public function indexAction()
    {
        $this->_title($this->__('Shipments'))->_title($this->__('Shipments MBE'));
        $this->loadLayout();
        $this->_setActiveMenu('sales/sales');
        $this->_addContent($this->getLayout()->createBlock('mbe_shipping/adminhtml_sales_shipment'));
        $this->renderLayout();
    }

    public function gridAction()
    {
        $this->loadLayout();
        $this->getResponse()->setBody(
            $this->getLayout()->createBlock('mbe_shipping/adminhtml_sales_shipment_grid')->toHtml()
        );
    }

    public function downloadWaybillAction()
    {
        $shipmentId = $this->getRequest()->getParam('shipment_id');

        /** @var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');
        $link = $helper->getShipmentUrlByShipmentId($shipmentId);

        Mage::app()->getResponse()->setRedirect($link)->sendResponse();
    }


    public function exportCsvAction()
    {
        $fileName = 'mbe_shipping.csv';
        $grid = $this->getLayout()->createBlock('mbe_shipping/adminhtml_sales_order_grid');
        $this->_prepareDownloadResponse($fileName, $grid->getCsvFile());
    }

    public function exportExcelAction()
    {
        $fileName = 'mbe_shipping.xml';
        $grid = $this->getLayout()->createBlock('mbe_shipping/adminhtml_sales_order_grid');
        $this->_prepareDownloadResponse($fileName, $grid->getExcelFile($fileName));
    }

    public function closeshipmentsAction()
    {
        $shipmentIds = $this->getRequest()->getParam('shipment_ids');

        if (!is_array($shipmentIds)) {
            $this->_getSession()->addError($this->__('Please select items.'));
        }
        else {
            $toClosedIds = array();
            $alreadyClosedIds = array();

            /** @var $helper Mbe_Shipping_Helper_Data */
            $helper = Mage::helper('mbe_shipping');

            /** @var  $ws Mbe_Shipping_Model_Ws */
            $ws = Mage::getModel('mbe_shipping/ws');


            foreach ($shipmentIds as $shipmentId) {
                if ($helper->isShippingOpen($shipmentId)) {

                    array_push($toClosedIds, $shipmentId);
                }
                else {
                    array_push($alreadyClosedIds, $shipmentId);
                }
            }

            $ws->closeShipping($toClosedIds);


            if (count($toClosedIds) > 0) {
                $this->_getSession()->addSuccess(
                    $this->__('Total of %d shipment(s) have been closed.', count($toClosedIds))
                );
            }

            if (count($alreadyClosedIds) > 1) {
                $this->_getSession()->addError(
                    $this->__('Total of %d shipment(s) was already closed.', count($alreadyClosedIds))
                );
            }
        }
        $this->_redirect('*/*/index');
    }
}